/**
 * 
 */
package be.spacebel.ssa.swe.beans;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.annotation.PostConstruct;

import swe.ssa.esa.swep.datamodel.common.MetadataType;

import be.spacebel.ssa.swe.data.OfferingMetadata;
import be.spacebel.ssa.swe.ui.UIFilter;
import be.spacebel.ssa.swe.util.FilterUtil;
import be.spacebel.ssa.swe.util.XMLUtil;
import be.spacebel.test.util.FacesMessageUtil;

/**
 * @author cnl
 * 
 */
public class SearchBean {
	public List<Test> getTests() {
		return tests;
	}

	public void setTests(List<Test> tests) {
		this.tests = tests;
	}

	private be.spacebel.ssa.swe.beans.MetadataBean metadataBean;
	private ArrayList<String> matchingUserDomain;
	private ArrayList<String> matchingDomain;
	private ArrayList<String> matchingProvider;
	private ArrayList<String> matchingAcquisition;
	private List<String> matchingOfferingNames;

	private List<Test> tests = new ArrayList<Test>();
	private ArrayList<String> matchingProperties;

	@PostConstruct
	public void init() {
		// execute update with an empty filter
		tests.add(new Test("name1", "value1"));
		tests.add(new Test("name2", "value2"));
		tests.add(new Test("name3", "value3"));
		update(new UIFilter());
	}

	/**
	 * update the matching offerings and filters when filter value is modified
	 * 
	 * @param filter
	 *            the modified filter
	 * 
	 */
	public void update(UIFilter filter) {
		this.matchingOfferingNames = new ArrayList<String>();
		this.matchingDomain = new ArrayList<String>();
		this.matchingUserDomain = new ArrayList<String>();
		this.matchingProvider = new ArrayList<String>();
		this.matchingAcquisition = new ArrayList<String>();
		this.matchingProperties = new ArrayList<String>();
		// update matching offerings
		for (OfferingMetadata offering : metadataBean.getOfferingList()) {
			if (FilterUtil.match(offering, filter)) {
				this.matchingOfferingNames.add(offering.getIdentifier());
			}
		}
		// update filter available values
		for (String offeringName : matchingOfferingNames
				.toArray(new String[matchingOfferingNames.size()])) {
			OfferingMetadata off = metadataBean.getOfferingDescriptions().get(
					offeringName);
			if (!this.matchingDomain.contains(off.getDomain())) {
				this.matchingDomain.add(off.getDomain());
			}
			if (!this.matchingUserDomain.contains(off.getUserDomain())) {
				this.matchingUserDomain.add(off.getUserDomain());
			}
			if (!this.matchingProvider.contains(off.getProvider().getName())) {
				this.matchingProvider.add(off.getProvider().getName());
			}
			if (!this.matchingProperties.containsAll(Arrays.asList(off
					.getObservablePropertyArray()))) {
				this.matchingProperties.addAll(Arrays.asList(off
						.getObservablePropertyArray()));
			}
			if (!this.matchingAcquisition.contains(off.getAcquisition())) {
				this.matchingAcquisition.add(off.getAcquisition());
			}
		}
		if(this.matchingOfferingNames.isEmpty()) {
			FacesMessageUtil.addWarningMessage("No results. Try to reset some filters.");
		}
		return;
	}

	public ArrayList<String> getMatchingProperties() {
		return matchingProperties;
	}

	public void setMatchingProperties(ArrayList<String> matchingProperties) {
		this.matchingProperties = matchingProperties;
	}

	public ArrayList<String> getMatchingUserDomain() {
		return matchingUserDomain;
	}

	public void setMatchingUserDomain(ArrayList<String> matchingUserDomain) {
		this.matchingUserDomain = matchingUserDomain;
	}

	public ArrayList<String> getMatchingDomain() {
		return matchingDomain;
	}

	public void setMatchingDomain(ArrayList<String> matchingDomain) {
		this.matchingDomain = matchingDomain;
	}

	public ArrayList<String> getMatchingProvider() {
		return matchingProvider;
	}

	public void setMatchingProvider(ArrayList<String> matchingProvider) {
		this.matchingProvider = matchingProvider;
	}

	public ArrayList<String> getMatchingAcquisition() {
		return matchingAcquisition;
	}

	public void setMatchingAcquisition(ArrayList<String> matchingAcquisition) {
		this.matchingAcquisition = matchingAcquisition;
	}

	public SearchBean() {
	}

	public be.spacebel.ssa.swe.beans.MetadataBean getMetadataBean() {
		return metadataBean;
	}

	public void setMetadataBean(
			be.spacebel.ssa.swe.beans.MetadataBean metadataBean) {
		this.metadataBean = metadataBean;
	}

	public List<String> getMatchingOfferingNames() {
		return matchingOfferingNames;
	}

	public void setMatchingOfferingNames(List<String> matchingOfferingNames) {
		this.matchingOfferingNames = matchingOfferingNames;
	}
}
