package be.spacebel.ssa.swe.data;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;

import org.apache.xmlbeans.XmlObject;

import be.spacebel.ssa.swe.util.SWEUtil;

import swe.ssa.esa.swep.datamodel.common.MetadataType.ContentDescription;
import swe.ssa.esa.swep.datamodel.common.MetadataType.ContentDescription.Field;
import swe.ssa.esa.swep.datamodel.common.ObservationOfferingDocument.ObservationOffering;
import swe.ssa.esa.swep.datamodel.common.ReferencesType;
import swe.ssa.esa.swep.datamodel.common.RelatedDatasetType;
import net.opengis.swes.x20.AbstractContentsType.Offering;

public class OfferingMetadata {

	
	public ArrayList<ReferencesMetadata> getReferences() {
		return references;
	}

	public void setReferences(ArrayList<ReferencesMetadata> references) {
		this.references = references;
	}

	private String description;
	private String identifier;
	private String procedure;
	private String[] observablePropertyArray;
	private Calendar firstRecordTimestamp;
	private Calendar lastRecordTimestamp;
	private HashMap<String, FieldMetadata> fields;
	private String acquisition;
	private ProviderMetadata provider;
	private String userDomain;
	private String domain;
	private String domainSubCatagory;
	private ArrayList<ReferencesMetadata> references;
	private ArrayList<RelatedDatasetMetadata> relatedDataset;
	
	private boolean textType=false;
	private boolean numType=false;
	private boolean chartType=false;
	private boolean mapType=false;
	private boolean picType=false;
	

	public OfferingMetadata(Offering offeringSource) {
		
		ObservationOffering offering = (ObservationOffering) offeringSource.getAbstractOffering().changeType(ObservationOffering.type);
		
		identifier = offering.getIdentifier();
		procedure = offering.getProcedure();
		observablePropertyArray = offering.getObservablePropertyArray();
		description = offering.getDescription();
		ContentDescription contentDesc = offering.getOfferingMetadata().getContentDescription();
		firstRecordTimestamp = contentDesc.getFirstRecordTimestamp();  
		setLastRecordTimestamp(contentDesc.getLastRecordTimestamp());
		fields = new HashMap<String,FieldMetadata>();
		for(Field field : contentDesc.getFieldArray()) {
			fields.put(field.getIdentifier(),new FieldMetadata(field));
		}
		setAcquisition(offering.getOfferingMetadata().getAcquisitionProcess().toString());
		setProvider(new ProviderMetadata(offering.getOfferingMetadata().getDataProvider()));
		//userDomain = offering.getOfferingMetadata().getUserDomainArray(0).get;
		setUserDomain(offering.getOfferingMetadata().getUserDomainArray(0).toString());
		setDomain(offering.getOfferingMetadata().getDomain().getName());
		setDomainSubCatagory(offering.getOfferingMetadata().getDomain().getSubCategory());
		references = new ArrayList<ReferencesMetadata>();
		for(ReferencesType  ref: offering.getOfferingMetadata().getReferencesArray()) {
		references.add(new ReferencesMetadata(ref));
		}
		relatedDataset = new ArrayList<RelatedDatasetMetadata>();
		for(RelatedDatasetType  related: offering.getOfferingMetadata().getRelatedDatasetTypeArray()) {
			relatedDataset.add(new RelatedDatasetMetadata(related));
		}
		
		assignTypes();
		
	}

	private void assignTypes() {
		for(FieldMetadata field : this.fields.values()) {
			if(field.getDataType().equalsIgnoreCase(SWEUtil.NUMERIC_TYPE)) {
				setNumType(true);
			}
			if(field.getDataType().equalsIgnoreCase(SWEUtil.TEXT_TYPE)) {
				setTextType(true);
			}
			if(field.getDataType().equalsIgnoreCase(SWEUtil.CHART_TYPE)) {
				setChartType(true);
			}
			if(field.getDataType().equalsIgnoreCase(SWEUtil.PICTURE_TYPE)) {
				setPicType(true);
			}
			if(field.getDataType().equalsIgnoreCase(SWEUtil.MAP_TYPE)) {
				setMapType(true);
			}
		}
		
	}

	public String getUserDomain() {
		return userDomain;
	}

	public void setUserDomain(String userdomain) {
		this.userDomain = userdomain;
	}

	
	public HashMap<String, FieldMetadata> getFields() {
		return fields;
	}

	public void setFields(HashMap<String, FieldMetadata> fields) {
		this.fields = fields;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public String getIdentifier() {
		return identifier;
	}

	public void setIdentifier(String identifier) {
		this.identifier = identifier;
	}

	public String getProcedure() {
		return procedure;
	}

	public void setProcedure(String procedure) {
		this.procedure = procedure;
	}

	public String[] getObservablePropertyArray() {
		return observablePropertyArray;
	}

	public void setObservablePropertyArray(String[] observablePropertyArray) {
		this.observablePropertyArray = observablePropertyArray;
	}

	public Calendar getFirstRecordTimestamp() {
		return firstRecordTimestamp;
	}

	public void setFirstRecordTimestamp(Calendar firstRecordTimestamp) {
		this.firstRecordTimestamp = firstRecordTimestamp;
	}

	public Calendar getLastRecordTimestamp() {
		return lastRecordTimestamp;
	}

	public void setLastRecordTimestamp(Calendar lastRecordTimestamp) {
		this.lastRecordTimestamp = lastRecordTimestamp;
	}

	public String getAcquisition() {
		return acquisition;
	}

	public void setAcquisition(String acquisition) {
		this.acquisition = acquisition;
	}

	public ProviderMetadata getProvider() {
		return provider;
	}

	public void setProvider(ProviderMetadata provider) {
		this.provider = provider;
	}

	public String getDomainSubCatagory() {
		return domainSubCatagory;
	}

	public void setDomainSubCatagory(String domainSubCatagory) {
		this.domainSubCatagory = domainSubCatagory;
	}

	public String getDomain() {
		return domain;
	}

	public void setDomain(String domain) {
		this.domain = domain;
	}

	public ArrayList<RelatedDatasetMetadata> getRelatedDataset() {
		return relatedDataset;
	}

	public void setRelatedDataset(ArrayList<RelatedDatasetMetadata> relatedDataset) {
		this.relatedDataset = relatedDataset;
	}

	public boolean isTextType() {
		return textType;
	}

	public void setTextType(boolean textType) {
		this.textType = textType;
	}

	public boolean isNumType() {
		return numType;
	}

	public void setNumType(boolean numType) {
		this.numType = numType;
	}

	public boolean isChartType() {
		return chartType;
	}

	public void setChartType(boolean chartType) {
		this.chartType = chartType;
	}

	public boolean isMapType() {
		return mapType;
	}

	public void setMapType(boolean mapType) {
		this.mapType = mapType;
	}

	public boolean isPicType() {
		return picType;
	}

	public void setPicType(boolean picType) {
		this.picType = picType;
	}
}
